defmodule DecentralisedBookIndex.Metadata.AuthorRole do
  use Ash.Resource,
    otp_app: :decentralised_book_index,
    domain: DecentralisedBookIndex.Metadata,
    data_layer: AshPostgres.DataLayer

  alias DecentralisedBookIndex.Metadata

  postgres do
    table "author_role"
    repo DecentralisedBookIndex.Repo
  end

  actions do
    defaults [:read, :update, :destroy]

    create :create do
      primary? true
      accept [:order, :role]
      argument :author_id, :uuid
    end
  end

  attributes do
    uuid_primary_key :id

    attribute :role, :string do
      allow_nil? true
      public? true
    end

    attribute :order, :integer do
      allow_nil? false
      public? true
    end

    timestamps()
  end

  relationships do
    belongs_to :book, Metadata.Book do
      allow_nil? true
    end

    has_one :author, Metadata.Author do
      allow_nil? false
    end
  end
end
